from django.shortcuts import render
from django.test.client import RequestFactory
from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework import status

from .models import Puppy
from .serializers import PuppySerializer

@api_view(['GET', 'DELETE', 'PUT'])
def get_delete_update_puppy(request, pk):
    try:
        puppy = Puppy.objects.get(pk=pk)
    except Puppy.DoesNotExist:
        return Response(status=status.HTTP_404_NOT_FOUND)

    # get details of a single puppy
    if request.method == 'GET':
        
        # return Response({})

        # we get the puppy using an ID. Run the tests to ensure they all pass.
        serializer = PuppySerializer(puppy)
        return Response(serializer.data)
    # delete a single puppy
    elif request.method == 'DELETE':
        # return Response({})
        puppy.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)

    # update details of a single puppy
    elif request.method == 'PUT':

        # return Response({})

        serializer = PuppySerializer(puppy, data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_204_NO_CONTENT)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

@api_view(['GET', 'POST'])
def get_post_puppies(request):
    # get all puppies
    if request.method == 'GET':

        # return Response({})

        puppies = Puppy.objects.all()
        serializer = PuppySerializer(puppies, many=True) # generate a nested list of 'edit' items
        return Response(serializer.data)
    # insert a new record for a puppy
    elif request.method == 'POST':

        # return Response({})

        data = {
            'name': request.data.get('name'),
            'age': int(request.data.get('age')),
            'breed': request.data.get('breed'),
            'color': request.data.get('color')
        }
        serializer = PuppySerializer(data=data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)